package com.huimv.manage.controller;

import com.huimv.manage.service.IEarmarkService;
import com.huimv.manage.util.Result;
import com.huimv.manage.webservice.task.EarmarkTask;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.UnsupportedEncodingException;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@CrossOrigin
@RestController
@Slf4j
@RequestMapping(value = "/eartag/earmark")
public class EarmarkController {
    @Value("${webservice.url}")
    private String webServiceUrl;
//    @Value("${webservice.producerId}")
//    private String producerId;
//    @Autowired
//    private Soap soap;
    @Autowired
    private EarmarkTask earmarkTask;
    @Autowired
    private IEarmarkService earmarkService;

    @RequestMapping(value = "/listEarmark",method = RequestMethod.GET)
    public Result listEarmark(@RequestParam(value = "pageSize") int pageSize){
        //
        return earmarkService.listEarmark(pageSize);
    }

    /**
     * @Method      : getEarmark
     * @Description : 下载新耳标号
     * @Params      : [applyId]
     * @Return      : com.huimv.manage.util.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/10/29       
     * @Time        : 22:58
     */
    @RequestMapping(value = "/getEarmark",method = RequestMethod.GET)
    public Result getEarmark(@RequestParam(value = "applyId") int applyId) throws UnsupportedEncodingException {
        //
        return earmarkService.getEarmark(applyId);
    }

    /**
     * @Method      : setUpdateDownState
     * @Description : 设置耳标号以下载状态
     * @Params      : [earmarkIDs]
     * @Return      : com.huimv.manage.util.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/10/29       
     * @Time        : 22:59
     */
    @RequestMapping(value = "/setUpdateDownState",method = RequestMethod.POST)
    public Result setUpdateDownState(@RequestParam(value = "earmarkIDs") String earmarkIDs) throws UnsupportedEncodingException {
        //
        return earmarkService.setUpdateDownState(earmarkIDs);
    }

    /**
     * @Method      : setProduceState
     * @Description : 设置生产状态
     * @Params      : [earmarkIDs]
     * @Return      : com.huimv.manage.util.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/10/29       
     * @Time        : 23:55
     */
    @RequestMapping(value = "/setProduceState",method = RequestMethod.POST)
    public Result setProduceState(@RequestParam(value = "ids") String ids,@RequestParam(value = "state") int state){
        //
        return earmarkService.setProduceState(ids,state);
    }

    /**
     * @Method      : setWriteState
     * @Description : 设置写入状态
     * @Params      : [ids, state]
     * @Return      : com.huimv.manage.util.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/10/30       
     * @Time        : 0:02
     */
    @RequestMapping(value = "/setWriteState",method = RequestMethod.POST)
    public Result setWriteState(@RequestParam(value = "ids") String ids,@RequestParam(value = "state") int state){
        //
        return earmarkService.setWriteState(ids,state);
    }

    /**
     * @Method      : setPrintState
     * @Description : 设置打码状态
     * @Params      : [ids, state]
     * @Return      : com.huimv.manage.util.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/10/30       
     * @Time        : 0:03
     */
    @RequestMapping(value = "/setPrintState",method = RequestMethod.POST)
    public Result setPrintState(@RequestParam(value = "ids") String ids,@RequestParam(value = "state") int state){
        //
        return earmarkService.setPrintState(ids,state);
    }


}
