package com.huimv.manage.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.huimv.manage.dao.entity.EtApplyEntity;
import com.huimv.manage.dao.repo.EtApplyRepo;
import com.huimv.manage.service.IApplyService;
import com.huimv.manage.util.Const;
import com.huimv.manage.util.Result;
import com.huimv.manage.util.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
public class ApplyServiceImpl implements IApplyService {

    @Autowired
    private EtApplyRepo applyRepo;

    /**
     * @Method      : listApply
     * @Description : 所有任务展示列表
     * @Params      : []
     * @Return      : com.huimv.manage.util.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/10/28       
     * @Time        : 18:20
     */
    @Override
    public Result listApply() {
        //
        return new Result(ResultCode.SUCCESS,applyRepo.listApply());
    }

    /**
     * @Method      : saveNewMission
     * @Description : 保存新任务（号）
     * @Params      : [applyJa]
     * @Return      : com.huimv.manage.util.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/10/28       
     * @Time        : 18:20
     */
    @Override
    public Result saveNewMission(JSONArray applyJa) {
        for(int a=0;a<applyJa.size();a++){
            JSONObject applyJo = applyJa.getJSONObject(a);
            EtApplyEntity applyEntity = new EtApplyEntity();
            applyEntity.setMissionProcessor(applyJo.getString("missionProcessor"));
            applyEntity.setOrgId(applyJo.getString("orgId"));
            applyEntity.setOrgName(applyJo.getString("orgName"));
            applyEntity.setOrgAddress(applyJo.getString("orgAddress"));
            applyEntity.setOrgLinkman(applyJo.getString("orgLinkman"));
            applyEntity.setPostCode(applyJo.getString("postCode"));
            applyEntity.setOrgTel(applyJo.getString("orgTel"));
            applyEntity.setApplyId(applyJo.getIntValue("applyId"));
            applyEntity.setAssignedDate(applyJo.getString("assignedDate"));
            applyEntity.setEarmarkAmount(applyJo.getIntValue("earmarkAmount"));
            applyEntity.setRealEarmarkAmount(applyJo.getIntValue("realEarmarkAmount"));
            applyEntity.setConsignmentDate(applyJo.getString("consignmentDate"));
            applyEntity.setApplyRegionName(applyJo.getString("applyRegionName"));
            applyEntity.setAnimalType(applyJo.getString("animalType"));
            applyEntity.setAnimalTypeName(applyJo.getString("animalTypeName"));
            applyEntity.setSetDownState(0);
            applyEntity.setDeliverState(0);
            applyRepo.saveAndFlush(applyEntity);
        }
        return new Result(ResultCode.SUCCESS, Const.SAVE_NEW_MESSION);
    }

    @Override
    public JSONArray filterExistApply(JSONArray downloadApplyJa) {
        JSONArray newApplyJa = new JSONArray();

        for(int a=0;a<downloadApplyJa.size();a++){
            JSONObject downApplyJo = downloadApplyJa.getJSONObject(a);
//            System.out.println("downApplyJo>>"+downApplyJo);
//            System.out.println("applyId>>"+downApplyJo.getIntValue("applyId"));
            EtApplyEntity applyEntity = new EtApplyEntity();
            applyEntity.setApplyId(downApplyJo.getIntValue("applyId"));
            ExampleMatcher matcher = ExampleMatcher.matching()
                    .withMatcher("apply_id" ,ExampleMatcher.GenericPropertyMatchers.contains());
            Example example = Example.of(applyEntity,matcher);
//            boolean applyExistBln = applyRepo.exists(example);
//            System.out.println("applyExistBln>>"+applyExistBln);
            if(!applyRepo.exists(example)){
                newApplyJa.add(downApplyJo);
            }
        }
        return newApplyJa;
    }

    /**
     * @Method      : setMissionHasDownloadState
     * @Description : 
     * @Params      : [applyId, state]
     * @Return      : com.huimv.manage.util.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/10/29       
     * @Time        : 15:36
     */
    @Override
    public Result setMissionHasDownloadState(int applyId, int state) {
        EtApplyEntity applyEntity = new EtApplyEntity();
        applyEntity.setApplyId(applyId);
        ExampleMatcher matcher = ExampleMatcher.matching()
                .withMatcher("apply_id" ,ExampleMatcher.GenericPropertyMatchers.contains());
        Example example = Example.of(applyEntity,matcher);
        Optional optional= applyRepo.findOne(example);
        if (optional.isPresent()){
            EtApplyEntity applyEntity1  = (EtApplyEntity) optional.get();
            applyEntity1.setSetDownState(state);
            applyEntity1.setSetDownDate(new Timestamp(new Date().getTime()));
            applyRepo.saveAndFlush(applyEntity1);
            return new Result(ResultCode.SUCCESS);
        }else{
            return new Result(Const.CODE_NO_RECORD,Const.NO_RECORD,false);
        }
    }
}
